// $Id: ffiles.C,v 1.1.1.1 2002/03/24 12:37:00 dvermeir Exp $
#include <stdexcept>
#include <dvutil/file.h>
#include <dvutil/tostring.h>
#include <dvnet/socket.h>

using namespace Dv;

int
add(Net::Socket& server, const string& path) {
// Recursively add a path to the index: if path is not
// a directory, we just send a single add command to the server,
// otherwise we recursively send such a command for each file in the
// directory.
int n(0);
Util::File f(path);
f.expand(); // make sure it is not a symbolic link
if (f.type() == Util::File::DIRECTORY) {
  try {
    Util::Directory d(f);
    const vector<string>& files(d.files());
    for (unsigned int i=0; i<files.size(); ++i)
      n += add(server, d.file(files[i]));
    return n;
    }
  catch (...) {
    return n;
    }
  }
server << "add " << path << endl;
string reply;
getline(server, reply);
return n + (reply == "OK" ? 1 : 0 );
}


int
main( int argc , char* argv[] ) {
static const string usage("usage: ffiles host port command");
static const string error("ERROR ");
try {
  // Get common parameters.
  if (argc<4)
    throw runtime_error(usage);

  const string host(argv[1]);
  const int port(atoi(argv[2]));
  const string cmd(argv[3]);

  // Connect to server.
  // Net::Socket server(argv[1],atoi(argv[2]),1024,5000);
  Net::Socket server(host, port);
  if (!server)
    throw runtime_error(Util::tostring(port) + "@" + host + ": " +
    		server.strerror());

  // Process command.

  if ( cmd=="stop" || cmd == "save" ) {
    string reply;
    server << cmd << endl;
    getline(server, reply);
    cout << reply << endl;
    }
  else if (cmd == "add") {
    if (argc<5)
      throw runtime_error(usage);
    const string path(argv[4]);
    cout << add(server, path) << " files added" << endl;
    }
  else if (cmd == "query") {
    if (argc<5)
      throw runtime_error(usage);
    const string query(argv[4]);
    server << cmd << ' ' << query << endl;
    string reply;
    while (server) {
      getline(server,reply);
      if (reply.size()==0)
        break;
      cout << reply << endl;
      }
    }
  else 
    throw runtime_error(cmd + ": bad command");
  }
catch (exception& e) {
  cerr << error <<  e.what() << endl;
  return 1;
  }
return 0;
}
