// $Id: index.C,v 1.1.1.1 2002/03/24 12:37:01 dvermeir Exp $
#include <vector>
// #include <limits>
#include <sstream>
#include <algorithm>
#include <dvutil/tostring.h>
#include <dvutil/date.h>
#include <config.h>
#include "index.h"
#include "word.h"
#include "wordstreamiterator.h"

Index::Index(size_t word_sz, istream* is): word_size_(word_sz) {
if (is)
  ignore(*is);
}

Index::~Index() {
}

static const char* MAGIC("#textindexer");

ostream&
operator<<(ostream& os, const Index& index) {
// Write first line: MAGIC version#
os << MAGIC << " " << VERSION << " -- do not edit this file" << endl;
// Write pool contents & remember the position of each pointer.
os << index.dates_.size() << endl;
map<const string*,int> file_nrs;
size_t n(0);
for (Index::Dates::const_iterator i = index.dates_.begin(); 
     i!=index.dates_.end(); ++i) {
  os << (*i).second << ' ' << *((*i).first) << "\n";
  file_nrs[(*i).first] = n++;
  }
for (Index::iterator i=index.begin(); i!= index.end(); ++i) {
  os << (*i).first;
  const Index::Files& files((*i).second);
  for (Index::Files::const_iterator j = files.begin(); j!=files.end(); ++j)
    os << " " << file_nrs[*j];
  os << "\n";
  }
return os;
}

istream&
operator>>(istream& is, Index& index) throw (runtime_error) {
// Read first line: MAGIC version junk
// Normally, we would use ignore(numeric_limits<int>::max(), '\n')
// but #include <limits> does not work in this g++ version.
string magic;
string junk;
double version;
is >> magic >> version;
getline(is,junk);
if (magic!=MAGIC)
  throw runtime_error("Bad magic in textindexer index file");
// Perhaps a test on version should come here.
vector<const string*> files;
StringPool& pool(index.pool());
size_t n;
is >> n;
string filename;
time_t t;
for (size_t i=0; i<n; ++i) {
  is >> t;
  if (t==0)
    throw runtime_error("Index reader: time == 0");
  is.ignore();
  getline(is, filename);
  if (filename.size()==0)
    throw runtime_error("Index reader: empty filename ");
  const string* pfn(pool[filename]);
  files.push_back(pfn);
  index.dates_[pfn] = t;
  }
string line;
while (getline(is,line)) {
  istringstream iss(line);
  Word w;
  iss >> w;
  size_t i;
  while (iss>>i) {
    if (i>=files.size())
      throw runtime_error(Dv::Util::tostring(i) + ": illegal file#");
    index.map_[w.str()].insert(files[i]);
    }
  }
return is;
}

time_t
Index::date(const string& fn) const {
const string* pfn(file(fn));
if (!pfn)
  return 0;
Dates::const_iterator i = dates_.find(pfn);
if (i==dates_.end())
  return 0;
return (*i).second;
}

const Index::Files*
Index::query(const string& s) const {
string ss(s,0,word_size());
Map::const_iterator i(map_.find(ss));
if (i==map_.end())
  return 0;
return &(*i).second;
}


Index::Files&
Index::query(const Strings& q, Index::Files& final_result) const {
Files	result[2];
int	r(0); // index of result
for (Strings::const_iterator i = q.begin(); i!=q.end(); ++i) {
  const Index::Files* qr(query(*i));
  if ( qr == 0) { // one of the strings not found: just return empty set
    final_result.clear();
    return final_result;
    }
  // assert qr != 0
  int nr((r+1)%2); // index of next result
  if (i==q.begin()) // first result, just store in result[nr]
    result[nr] = *qr;
  else {
    result[nr].clear();
    set_intersection(result[r].begin(), result[r].end(), 
    		     qr->begin(), qr->end(),
		     inserter(result[nr],result[nr].end()));
    }
  r = nr;
  }
final_result = result[r];
return final_result;
}
  
size_t
Index::insert(const string& fn, istream& is) {
const string* pfn(file(fn));
if (pfn)
  remove(fn);
pfn = pool_[fn];
size_t n(0);
wordstream_iterator i(is,ignore());
wordstream_iterator end;
for (; i!=end; ++i) {
  string w(*i,0,word_size());
  n += (map_[w].insert(pfn).second ? 1 : 0);
  }
dates_[pfn] = Dv::Util::Date().time();
return n;
}

void
Index::remove(const string& fn) {
const string* pfn(file(fn));
if (pfn == 0)
  return; 
for (Map::iterator i = map_.begin(); i!=map_.end(); ++i) {
  Files& files((*i).second);
  files.erase(pfn);
  if (files.size() == 0)
    map_.erase(i);
  }
dates_.erase(pfn);
pool_.remove(fn);
}

void
Index::ignore(const string& w) {
ignore_.insert(w);
}

void
Index::ignore(istream& is) {
Word w;
while (is>>w)
 ignore(w);
}
