// $Id: query.C,v 1.2 2002/03/27 11:21:21 dvermeir Exp $
#include <sstream>
#include <vector>
#include <iterator>
#include <fstream>
#include <algorithm>
#include "query.h"
#include "wordstreamiterator.h"

Query::Query(const Index& index, const string &q) throw (runtime_error): index_(index), query_(q) {
static const string syntax_error("SYNTAX ERROR "); 
// First fill words_, it will be used to query the index_.
istringstream iss(q);
copy(wordstream_iterator(iss, index.ignore()), 
     wordstream_iterator(),
     inserter(words_, words_.end()));
if (words_.size() == 0)
  throw runtime_error(syntax_error + q);
// Next detect phrases, these will be used to check candidate files.
string::size_type pos(0);
string::size_type start(string::npos);
string::size_type end(string::npos);
while ((start = q.find('"',pos)) != string::npos) {
  // Got a double quote at position start.
  // First make 1-word phrases out of q[pos]..q[start].
  {
  string words(q,pos, start - pos);
  istringstream words_iss(words);
  wordstream_iterator i(words_iss, index.ignore());
  wordstream_iterator end;
  for ( ; i!=end; ++i) {
    string w(*i);
    // only necessary of word is longer than index word_size.
    if (w.size() > index.word_size())
      phrases_.push_back(Phrase(1,w));
    }
  }
  // Find closing double quote.
  end = q.find('"', start+1);
  if (end == string::npos)  // No closing double quote.
    throw runtime_error(syntax_error + q);
  // Retrieve phrase string.
  string phrase_s(q, start + 1, end - start -1);
  istringstream iss(phrase_s);
  Phrase phrase;
  copy(wordstream_iterator(iss, index.ignore()), 
       wordstream_iterator(),
       back_inserter(phrase));
  phrases_.push_back(phrase);
  pos = end+1;
  }
// Make 1-word phrases out of tail q[pos..] of string.
{
string words(q,pos);
istringstream words_iss(words);
wordstream_iterator i(words_iss, index.ignore());
wordstream_iterator end;
for ( ; i!=end; ++i) {
  string w(*i);
  // only necessary of word is longer than index word_size.
  if (w.size() > index.word_size())
    phrases_.push_back(Phrase(1,w));
  }
}
}

const Index::Files&
Query::exec() {
// 1. Retrieve list of files containing all (short versions of) words.
Index::Files	fset;
index_.query(words_, fset);
// 2. For each file in fset, check if all phrases occur. If so,
//    add it to result.
for (Index::Files::iterator f=fset.begin(); f!=fset.end(); ++f) {
  // Grab a copy of the file in a vector. This avois repeatedly
  // reading the file, hopefully improving performance.
  vector<string> fseq;
  ifstream ifs((**f).c_str());
  if (!ifs)
    continue; // Next file.
  copy(wordstream_iterator(ifs, index_.ignore()), 
       wordstream_iterator(),
       back_inserter(fseq));
  // Check that each phrase appears in fseq.
  bool ok(true);
  for (list<Query::Phrase>::const_iterator p = phrases_.begin(); 
       ok && p!=phrases_.end(); 
       ++p) {
    ok = (search(fseq.begin(),fseq.end(),p->begin(),p->end()) != fseq.end());
    }
  if (ok)
    result_.insert(*f);
  }
return result_;
}

ostream&
operator<<(ostream& os, const Query& q) {
os << "q.words = ";
copy(q.words_.begin(), q.words_.end(), ostream_iterator<string>(os," "));
os << "\n";
os << "q.phrases = " << endl;
for (list<Query::Phrase>::const_iterator i=q.phrases_.begin(); 
     i!=q.phrases_.end(); ++i) {
  os << " \"";
  copy((*i).begin(), (*i).end(), ostream_iterator<string>(os," "));
  os << "\"\n";
  }
return os;
}

