#ifndef WORD_H
#define WORD_H

// $Id: word.h,v 1.1.1.1 2002/03/24 12:37:04 dvermeir Exp $
#include <string>
#include <iostream>
#include <stdexcept>

/**
 * A word is a sequence of lower case alphabetic characters.
 * The only way to obtain a non-empty wordt is via the input operator
 * operator<<(istream&, Word&).
 */
class Word {
public:
  /**
   * Constructor.
   * \param w initial underlying string.
   */
  Word(const string& w="");
  /**
   * Convert to string.
   */
  operator string() const { return w_; }
  /**
   * \return reference to underlying string.
   */
  const string& str() const { return w_; }
  /**
   * Read a word from a stream. Non-alhabetic characters before and
   * after the word will be skipped.
   * \param is stream to read from.
   * \param w word that will be read (it is cleared beforehand).
   * \return reference to \a is.
   */
  friend istream&
    operator>>(istream&, Word& w);
  friend bool
    operator==(const Word& w1, const Word& w2) { return w1.w_ == w2.w_; }
  friend bool
    operator<(const Word& w1, const Word& w2) { return w1.w_ < w2.w_; }
  /** \example test-word.C 
   * The following program tests the Word class.
   */
private:
  string w_;
};

#endif

