#ifndef WORDSTREAMITERATOR_H
#define WORDSTREAMITERATOR_H

// $Id: wordstreamiterator.h,v 1.1.1.1 2002/03/24 12:37:04 dvermeir Exp $
#include <string>
#include <iostream>
#include <stdexcept>
#include <iterator>
#include <set>


/**
 * A wordstream_iterator is an input iterator that returns words from
 * an underlying stream.
 * The words are guaranteed to conform to the class Word and do not
 * appear in an ignore set of common words.
 */
class wordstream_iterator: input_iterator<string,ptrdiff_t> {
public:
  /**
   * Constructor.
   * \param is stream to read from
   * \param ignore set of strings to be ignored
   */
  wordstream_iterator(istream& is, const set<string>& ignore);
  wordstream_iterator();

  const string& operator*() const { return value_; }
  wordstream_iterator& operator++() { get(); return *this; }
  wordstream_iterator operator++(int) { 
    wordstream_iterator tmp(*this);
    get();
    return tmp;
    }
  friend bool
  operator==(const wordstream_iterator& i1, const wordstream_iterator& i2) {
    return i1.is_ == i2.is_ && i1.ok_ == i2.ok_ 
        || i1.ok_ == false && i2.ok_ == false;
    }

  size_t ignored() const { return ignored_; }
  size_t read() const { return read_; }
private:
  void get();

  const set<string>* ignore_;
  string value_;
  istream* is_;
  bool ok_;
  size_t ignored_;
  size_t read_;
};
#endif

